import type { TObject, Static } from '@sinclair/typebox';
import type { QuerySchema, WhereParam } from './types';
import { BaseView } from './BaseView';
import { SqlExecutor } from './sql';
export declare abstract class BaseTable<T extends TObject, C> extends BaseView<T, C> {
    protected abstract _EXECUTOR: SqlExecutor<T>;
    /**
     * Auto convert data
     * check row data while insert or update
     * */
    private checkEntity;
    deleteByField(field: string, value: string | number | boolean): Promise<number>;
    deleteByQuery(query: QuerySchema): Promise<number>;
    deleteByCondition(condition: WhereParam): Promise<number>;
    /**
     * Delete a row by primary id
    */
    deleteById(id: number | string): Promise<number>;
    /**
     * Update a record, By Primary Key in the obj
    */
    update(obj: Static<T>): Promise<number>;
    updateByField(obj: Static<T>, field: string, value: string | number | boolean): Promise<number>;
    updateByQuery(obj: Static<T>, query: QuerySchema): Promise<number>;
    updateByCondition(obj: Static<T>, condition?: WhereParam): Promise<number>;
    /**
     * Insert a record
    */
    add(object: Static<T>): Promise<Static<T>>;
}
