"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseTable = void 0;
const lodash_1 = __importDefault(require("lodash"));
const BaseView_1 = require("./BaseView");
const QueryBuilder_1 = require("./QueryBuilder");
const dayjs_1 = __importDefault(require("dayjs"));
const toDate = (txt) => {
    return (0, dayjs_1.default)(txt).toDate();
};
class BaseTable extends BaseView_1.BaseView {
    /**
     * Auto convert data
     * check row data while insert or update
     * */
    checkEntity(obj, isAdd = false) {
        // checkEntity(this.schema)
        let clone = {};
        this._CONFIG.FIELD_MAP.forEach((schema, key) => {
            let field = schema.column || key;
            if (lodash_1.default.has(obj, key))
                clone[field] = obj[key];
            let type = (0, QueryBuilder_1.getFieldType)(schema);
            if (type == 'date') {
                if (schema.isCreate) {
                    if (isAdd) {
                        clone[field] = new Date();
                    }
                    else {
                        lodash_1.default.unset(clone, field);
                    }
                    return;
                }
                if (schema.isModify) {
                    clone[field] = new Date();
                    return;
                }
                clone[field] = toDate(obj[key]);
            }
        });
        console.log(clone);
        return clone;
    }
    deleteByField(field, value) {
        const { _CONFIG: { mark, FIELD_MAP } } = this;
        if (mark)
            return this.updateByField(mark, field, value);
        let schema = FIELD_MAP.get(field);
        let column = (schema && schema.column) ? schema.column : field;
        return this.deleteByCondition([{ column, value }]);
    }
    deleteByQuery(query) {
        const { _CONFIG: { mark } } = this;
        if (mark)
            return this.updateByQuery(mark, query);
        const condition = (0, QueryBuilder_1.queryToCondition)(query, this._CONFIG.FIELD_MAP, this._QUERY_CACHE);
        return this.deleteByCondition(condition);
    }
    deleteByCondition(condition) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { mark } } = this;
        if (mark)
            return this.updateByCondition(mark, condition);
        const SQL = _BUILDER.delete(_table);
        const [WHERE, PARAM] = this._BUILDER.where(condition);
        return _EXECUTOR.execute(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, PARAM);
    }
    /**
     * Delete a row by primary id
    */
    deleteById(id) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { key, mark } } = this;
        if (key == null)
            throw new Error(`Table ${_table} do not have a Primary Key`);
        if (mark)
            return this.update(Object.assign(Object.assign({}, mark), { [key]: id }));
        const SQL = _BUILDER.delete(_table);
        const [WHERE, PARAM] = _BUILDER.byField(key, id);
        return _EXECUTOR.execute(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, PARAM);
    }
    /**
     * Update a record, By Primary Key in the obj
    */
    update(obj) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { key } } = this;
        // if (key == null) throw new Error(`Table ${_table} do not have a Primary Key`);
        if (!lodash_1.default.has(obj, key))
            throw new Error(`Update Action must have a key`);
        let entity = this.checkEntity(obj, false);
        const [SQL, FIELD_SET] = _BUILDER.update(_table, entity, key);
        if (FIELD_SET.length == 0) {
            throw new Error(`Update Action must have some properties`);
        }
        const [WHERE, PARAM] = _BUILDER.byField(key, obj[key], FIELD_SET.length + 1);
        return _EXECUTOR.execute(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, [...FIELD_SET, ...PARAM]);
    }
    updateByField(obj, field, value) {
        let schema = this._CONFIG.FIELD_MAP.get(field);
        let column = (schema && schema.column) ? schema.column : field;
        return this.updateByCondition(obj, [{ column, value }]);
    }
    updateByQuery(obj, query) {
        const condition = (0, QueryBuilder_1.queryToCondition)(query, this._CONFIG.FIELD_MAP, this._QUERY_CACHE);
        return this.updateByCondition(obj, condition);
    }
    updateByCondition(obj, condition) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { key } } = this;
        lodash_1.default.unset(obj, key);
        let entity = this.checkEntity(obj, false);
        if (lodash_1.default.keys(entity).length == 0)
            return new Promise(r => r(0));
        const [SQL, FIELD_SET] = _BUILDER.update(_table, entity);
        const [WHERE, PARAM] = this._BUILDER.where(condition, FIELD_SET.length + 1);
        return _EXECUTOR.execute(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, [...FIELD_SET, ...PARAM]);
    }
    /**
     * Insert a record
    */
    add(object) {
        const { _table, _BUILDER, _EXECUTOR } = this;
        let entity = this.checkEntity(object, true);
        const [SQL, PARAM] = _BUILDER.insert(_table, entity);
        return _EXECUTOR.add(this.getClient(), `${SQL}`, PARAM);
    }
}
exports.BaseTable = BaseTable;
