import type { TObject, Static } from '@sinclair/typebox';
import type { QuerySchema, WhereParam, WhereDefine, USchema, WhereItem, Sort } from './types';
import { BaseQuery } from './BaseQuery';
export type TableOptions = {
    /**
     * primary key Column Default "id"
     * if not , must specfy key field
    */
    key?: string;
    /**
     * The Table's Default Sorting Rule : Order Field , By Method
    */
    sort?: Sort;
    /**
     * 1. PageSize , if not specified , use DEFAULT_PAGESIZE
     * 2. DEFAULT_PAGESIZE can use setup to specified default : 10
    */
    pageSize?: number;
    /**
     * 默认查询过滤,比如 {field:'disabled',value:0,operation:'<>'}
     * 设置后，通过 query / all 拼装的 sql 都会带上 AND disabled <> 0
    */
    globalCondition?: WhereItem[];
};
export declare abstract class BaseView<T extends TObject, C> extends BaseQuery<C> {
    protected _table: string;
    protected _F2C: Map<string, string>;
    protected _C2F: Map<string, string>;
    protected _CONFIG: {
        key: string;
        sort: Sort;
        mark: any;
        pageSize: number;
        FIELD_MAP: Map<string, USchema>;
        fields_query: string;
        fields_get: string;
        WHERE_FIX: [string, string];
    };
    protected _QUERY_CACHE: Map<string, WhereDefine>;
    /**
     * @param tableName Data table name, "${schemaName}.${tableName}"
     *  "${schemaName}." can be ignore with the default search_path.
     * @param schema The Object Schema, oor will not validate the value
     * @param options (Table/View) Options
     *
    */
    constructor(tableName: string, schema: T, options?: TableOptions);
    protected abstract init(schema: T, options?: TableOptions): void;
    private _query;
    /**
     * @see WhereCondition
     * Use a WhereCondition Query Data
    */
    queryByCondition(condition?: WhereParam, limit?: QuerySchema): Promise<Static<T>[]>;
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data
    */
    query(query?: QuerySchema): Promise<Static<T>[]>;
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data With Page
     * this will return a object with {total:number,list:ArrayLike<T>}
    */
    queryPager(query?: QuerySchema): Promise<{
        total: number;
        list: Static<T>[];
    }>;
    /**
     * Fetch All Records form the Table / View
    */
    all(): Promise<Static<T>[]>;
    /**
     * Get A record form Table / View By Primary key.
     * This method will return All column. Even if the IGNORE column.
    */
    getById(id: number | string): Promise<Static<T>>;
    /**
     * Get A record form Table / View By Specify Field = value.
     * This method will return All column. Even if the IGNORE column.
     * Note : If result has multi records , return the first row
     *        Want return all records?  use `queryByField`
    */
    getByField(field: string, value?: string | number): Promise<Static<T>>;
    /**
     * Get records form Table / View By Specify Property = value.
    */
    queryByField(field: string, value?: string | number | boolean): Promise<Static<T>[]>;
    protected fixWhere(where: string): string;
    private orderByLimit;
}
