"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseView = void 0;
const lodash_1 = __importDefault(require("lodash"));
const Util_1 = require("./Util");
const BaseQuery_1 = require("./BaseQuery");
const QueryBuilder_1 = require("./QueryBuilder");
const GLOBAL_ID_FIELD = new Set(['id', 'guid']);
// const WHERE_TO_STRING = ();
class BaseView extends BaseQuery_1.BaseQuery {
    /**
     * @param tableName Data table name, "${schemaName}.${tableName}"
     *  "${schemaName}." can be ignore with the default search_path.
     * @param schema The Object Schema, oor will not validate the value
     * @param options (Table/View) Options
     *
    */
    constructor(tableName, schema, options) {
        super();
        this._F2C = new Map(); // Field To Column
        this._C2F = new Map(); // Column To Field
        this._CONFIG = {
            key: null,
            sort: null,
            mark: null,
            pageSize: Util_1.PAGE_SIZE,
            FIELD_MAP: null,
            fields_query: '*',
            fields_get: '*',
            WHERE_FIX: ['', ' WHERE '],
        };
        this._QUERY_CACHE = new Map();
        this._table = tableName;
        this._CONFIG.FIELD_MAP = new Map();
        let SortGuess = [];
        let field = null, by = 'desc';
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            let column = properties.column || field;
            this._F2C.set(field, column);
            this._C2F.set(column, field);
            this._CONFIG.FIELD_MAP.set(field, properties);
            if (GLOBAL_ID_FIELD.has(column) && this._CONFIG.key == null) {
                this._CONFIG.key = column;
                SortGuess.push(field);
            }
            ;
            if (properties.isModify)
                SortGuess.unshift(field);
            if (lodash_1.default.has(properties, 'delMark') && properties.delMark != null) {
                this._CONFIG.mark = { [column]: properties.delMark };
            }
        });
        if (options != null) {
            if (options.pageSize)
                this._CONFIG.pageSize = options.pageSize;
            if (options.key) {
                this._CONFIG.key = options.key;
                SortGuess.push(this._CONFIG.key);
            }
            if (options.sort) {
                field = options.sort.order;
                by = options.sort.by;
            }
        }
        if (field == null && SortGuess.length)
            field = SortGuess[0];
        if (field != null) {
            let column = this._F2C.has(field) ? this._F2C.get(field) : (this._C2F.has(field) ? field : null);
            if (column) {
                this._CONFIG.sort = { order: column, by };
            }
        }
        this.init(schema, options);
    }
    ;
    _query(WHERE, PARAM = [], ORDER_BY = '', LIMIT = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const { _BUILDER, _EXECUTOR, _table, _CONFIG: { fields_query } } = this;
            const SQL_QUERY = _BUILDER.select(_table, fields_query);
            const SQL = `${SQL_QUERY} ${WHERE} ${ORDER_BY} ${LIMIT}`;
            const result = _EXECUTOR.query(this.getClient(), SQL, PARAM);
            return result;
        });
    }
    /**
     * @see WhereCondition
     * Use a WhereCondition Query Data
    */
    queryByCondition(condition, limit) {
        const [WHERE, PARAM] = this._BUILDER.where(condition);
        if (limit) {
            const [ORDER_BY, LIMIT] = this.orderByLimit(limit);
            return this._query(this.fixWhere(WHERE), PARAM, ORDER_BY, LIMIT);
        }
        return this._query(this.fixWhere(WHERE), PARAM);
    }
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data
    */
    query(query) {
        const { _QUERY_CACHE, _CONFIG: { FIELD_MAP } } = this;
        const condition = (0, QueryBuilder_1.queryToCondition)(query, FIELD_MAP, _QUERY_CACHE);
        return this.queryByCondition(condition, query);
    }
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data With Page
     * this will return a object with {total:number,list:ArrayLike<T>}
    */
    queryPager(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let total = 0;
            const { _table, _BUILDER, _EXECUTOR, _QUERY_CACHE, _CONFIG: { FIELD_MAP } } = this;
            const condition = (0, QueryBuilder_1.queryToCondition)(query, FIELD_MAP, _QUERY_CACHE);
            const [WHERE, PARAM] = _BUILDER.where(condition);
            if (lodash_1.default.has(query, 'total_') && lodash_1.default.isNumber(query.total_)) {
                total = query.total_;
            }
            else {
                const SQL_COUNT = `${_BUILDER.count(_table)} ${this.fixWhere(WHERE)}`;
                const countResult = yield _EXECUTOR.get(this.getClient(), SQL_COUNT, PARAM);
                if (countResult == null) {
                    return {
                        total: 0,
                        list: [],
                    };
                }
                total = parseInt(countResult.total);
            }
            const [ORDER_BY, LIMIT] = this.orderByLimit(query);
            const list = yield this._query(this.fixWhere(WHERE), PARAM, ORDER_BY, LIMIT);
            return { total, list };
        });
    }
    /**
     * Fetch All Records form the Table / View
    */
    all() {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { fields_query, WHERE_FIX } } = this;
        const SQL = _BUILDER.select(_table, fields_query) + WHERE_FIX[0];
        return _EXECUTOR.query(this.getClient(), SQL);
    }
    /**
     * Get A record form Table / View By Primary key.
     * This method will return All column. Even if the IGNORE column.
    */
    getById(id) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { key, fields_get } } = this;
        if (key == null)
            throw new Error(`Table ${_table} do not have a Primary Key`);
        const SQL = _BUILDER.select(_table, fields_get);
        const [WHERE, PARAM] = _BUILDER.byField(key, id);
        return _EXECUTOR.get(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, PARAM);
    }
    /**
     * Get A record form Table / View By Specify Field = value.
     * This method will return All column. Even if the IGNORE column.
     * Note : If result has multi records , return the first row
     *        Want return all records?  use `queryByField`
    */
    getByField(field, value) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { fields_get } } = this;
        const SQL = _BUILDER.select(_table, fields_get);
        const [WHERE, PARAM] = _BUILDER.byField(field, value);
        return _EXECUTOR.get(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, PARAM);
    }
    /**
     * Get records form Table / View By Specify Property = value.
    */
    queryByField(field, value) {
        const { _table, _BUILDER, _EXECUTOR, _CONFIG: { fields_get } } = this;
        const SQL = _BUILDER.select(_table, fields_get);
        const [WHERE, PARAM] = _BUILDER.byField(field, value);
        return _EXECUTOR.query(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, PARAM);
    }
    fixWhere(where) {
        const { _CONFIG: { WHERE_FIX } } = this;
        let whereStr = lodash_1.default.trim(where);
        return whereStr.length ? (WHERE_FIX[0] + WHERE_FIX[1] + whereStr) : WHERE_FIX[0];
    }
    orderByLimit(query) {
        const { _BUILDER, _C2F, _F2C, _CONFIG: { sort, pageSize } } = this;
        return [
            _BUILDER.orderBy(_F2C, _C2F, query, sort),
            _BUILDER.limit(query, pageSize)
        ];
    }
}
exports.BaseView = BaseView;
