"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertDate = exports.setFormat = void 0;
const dayjs_1 = __importDefault(require("dayjs"));
const utc_1 = __importDefault(require("dayjs/plugin/utc"));
const timezone_1 = __importDefault(require("dayjs/plugin/timezone"));
dayjs_1.default.extend(utc_1.default);
dayjs_1.default.extend(timezone_1.default);
const DEFAULT_TIMEZONE = dayjs_1.default.tz.guess();
// export const tz=>{}
// https://day.js.org/docs/en/display/as-iso-string
// ISO 8601
// https://www.iana.org/time-zones
// https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
// https://github.com/iamkun/dayjs/blob/dev/src/plugin/utc/index.js
// https://github.com/iamkun/dayjs/blob/dev/src/plugin/timezone/index.js
// https://stackoverflow.com/questions/58847869/utc-vs-iso-format-for-time
const TZ_MAP = new Map([
    ['Etc/GMT+12', 'YYYY/MM/DD HH:mm:ss'],
    ['America/Mexico_City', 'MM/DD/YYYY HH:mm:ss'],
    ['Asia/Shanghai', 'YYYY-MM-DD HH:mm:ss'],
]);
const setFormat = (timezone, formater) => {
    TZ_MAP.set(timezone, formater);
};
exports.setFormat = setFormat;
// 
const convertDate = (date, timezone = DEFAULT_TIMEZONE) => {
    return (0, dayjs_1.default)(date).tz(timezone).format(TZ_MAP.get(timezone));
};
exports.convertDate = convertDate;
