"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryToCondition = exports.getFieldType = void 0;
const lodash_1 = __importDefault(require("lodash"));
const typebox_1 = require("@sinclair/typebox");
const types_1 = require("./types");
const Util_1 = require("./Util");
const DEFAULT_QUERY_KEY = new Set(['start_', 'count_', 'order_', 'by_', 'keyword_']);
const getFieldType = (schema) => {
    // union date type : string/number/date
    if (schema.__DATE)
        return 'date';
    switch (schema[typebox_1.Kind]) {
        case 'String':
            return 'string';
        case 'Number':
            return 'number';
        case 'Integer':
            return 'int';
        case 'Boolean':
            return 'boolean';
        case 'Date':
            return 'date';
        default:
            return 'string';
    }
};
exports.getFieldType = getFieldType;
/**
 * Cache Query Field To Where Filed
*/
const fieldToDef = (key, FIELD_MAP) => {
    let SCHEMA = FIELD_MAP.get(key);
    let query_field = key, suffix = null;
    if (SCHEMA == null) {
        for (let fix of types_1.SUFFIX) {
            if (key.endsWith(fix)) {
                let temp = key.substring(0, key.length - fix.length);
                if (FIELD_MAP.has(temp)) {
                    query_field = temp;
                    SCHEMA = FIELD_MAP.get(temp);
                    suffix = fix;
                    break;
                }
            }
        }
    }
    if (SCHEMA == null) {
        return null;
    }
    const fieldType = (0, exports.getFieldType)(SCHEMA);
    // @ts-ignore // field only use in inner , use for Query Buider validate
    let def = { field: query_field, column: SCHEMA.column || query_field, type: fieldType, fn: suffix };
    return def;
};
const defineToItem = (def, schema, value) => {
    // TODO : IGNORE SOME INVALIDATE SCHEMA
    // return null;
    return Object.assign(Object.assign({}, def), { value });
};
const queryToCondition = (query, FIELD_MAP, FIELD_CACHE) => {
    const err = [];
    const ROOT = { link: 'AND', items: [] };
    lodash_1.default.keys(query).map(key => {
        let define = null;
        if (FIELD_CACHE.has(key)) {
            define = FIELD_CACHE.get(key);
            if (define == null)
                return; // ignore default and error condition
        }
        else if (DEFAULT_QUERY_KEY.has(key)) {
            FIELD_CACHE.set(key, null);
            return;
        }
        ;
        if (define == null) {
            define = fieldToDef(key, FIELD_MAP);
        }
        FIELD_CACHE.set(key, define);
        if (define == null) {
            err.push(key);
            return;
        }
        ;
        // @ts-ignore
        let queryItem = defineToItem(define, FIELD_MAP.get(define.column || define.field), query[key]);
        if (queryItem == null) {
            err.push(key + ' \' value has problem : ' + String(query[key]));
            return;
        }
        ROOT.items.push(queryItem);
    });
    (0, Util_1.throwErr)(err, 'Some SQL Error Occur');
    return ROOT;
};
exports.queryToCondition = queryToCondition;
