"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.boolValue = exports.inNumber = exports.inString = exports.betweenDate = exports.betweenNumber = exports.checkEntity = exports.throwErr = exports.UType = exports.setup = exports.PAGE_SIZE = exports.ShowSql = exports.NONE_PARAM = void 0;
const lodash_1 = __importDefault(require("lodash"));
const dayjs_1 = __importDefault(require("dayjs"));
const typebox_1 = require("@sinclair/typebox");
const value_1 = require("@sinclair/typebox/value");
const Providers_1 = require("./Providers");
exports.NONE_PARAM = new Set(['IsNull', 'NotNull']);
var STRICT_QUERY = false;
var STRICT_ENTITY = false;
exports.ShowSql = null;
exports.PAGE_SIZE = 10;
const setup = (settings) => {
    if (lodash_1.default.isArray(settings.provider)) {
        (0, Providers_1.setProvider)(settings.provider[0], settings.provider[1]);
    }
    else if (lodash_1.default.isFunction(settings.provider)) {
        (0, Providers_1.setProvider)('pg', settings.provider);
    }
    if (lodash_1.default.has(settings, 'strict')) {
        if (settings.strict) {
            if (lodash_1.default.isBoolean(settings.strict)) {
                STRICT_QUERY = true;
                STRICT_ENTITY = true;
            }
            else {
                if (settings.strict.entity)
                    STRICT_ENTITY = true;
                if (settings.strict.query)
                    STRICT_QUERY = true;
            }
        }
        else {
            STRICT_QUERY = false;
            STRICT_ENTITY = false;
        }
    }
    if (settings.pageSize)
        exports.PAGE_SIZE = settings.pageSize;
    if (settings.showSQL && lodash_1.default.isFunction(settings.showSQL))
        exports.ShowSql = settings.showSQL;
};
exports.setup = setup;
exports.UType = {
    Table: (properties) => typebox_1.Type.Partial(typebox_1.Type.Object(properties)),
    Number: (options) => typebox_1.Type.Number(options),
    String: (options) => typebox_1.Type.String(options),
    Date: (options) => typebox_1.Type.Union([typebox_1.Type.Date(options), typebox_1.Type.String(), typebox_1.Type.Number()], Object.assign(Object.assign({}, options), { __DATE: true })),
    Boolean: (options) => typebox_1.Type.Boolean(options),
    Integer: (options) => typebox_1.Type.Integer(options),
};
const throwErr = (err, message) => {
    if (err.length == 0)
        return;
    if (!STRICT_QUERY) {
        console.error(message + '\n      ' + err.join('\n      '));
        return;
    }
    throw new Error(message ? message : err[0], { cause: err.join('\n') });
};
exports.throwErr = throwErr;
const checkEntity = (T, val) => {
    if (!STRICT_ENTITY) {
        return;
    }
    const result = value_1.Value.Errors(T, val);
    let err = result.next();
    if (err) {
        throw new Error('Entity Has Some value');
    }
};
exports.checkEntity = checkEntity;
const getStart = (str) => {
    if (str.startsWith('('))
        return ['>', str.substring(1)];
    if (str.startsWith('['))
        return ['>=', str.substring(1)];
    return ['>=', str];
};
const getEnd = (str) => {
    if (str.endsWith(')'))
        return ['<', str.substring(0, str.length - 1)];
    if (str.endsWith(']'))
        return ['<=', str.substring(0, str.length - 1)];
    return ['<=', str];
};
const between = (txt) => {
    let str = lodash_1.default.trim((txt || ''));
    if (str.length == 0)
        return null;
    let ptns = str.split(',').map(lodash_1.default.trim);
    if (ptns.length > 2)
        return null;
    if (ptns.length == 1)
        return [getStart(ptns[0])];
    let result = [];
    if (ptns[0].length)
        result.push(getStart(ptns[0]));
    if (ptns[1].length)
        result.push(getEnd(ptns[1]));
    return result;
};
const betweenNumber = (txt, parseFn) => {
    let range = between(txt);
    if (range == null)
        return null;
    let result = [];
    for (let item of range) {
        try {
            let num = parseFn(item[1]);
            result.push([item[0], num]);
        }
        catch (_a) {
            return null;
        }
    }
    return result;
};
exports.betweenNumber = betweenNumber;
const betweenDate = (txt) => {
    let range = between(txt);
    if (range == null)
        return null;
    let result = [];
    for (let item of range) {
        let day = (0, dayjs_1.default)(item[1]);
        if (!day.isValid()) {
            return null;
        }
        result.push([item[0], day.toDate()]);
    }
    return result;
};
exports.betweenDate = betweenDate;
const inString = (txt) => {
    return txt.split(',').map(lodash_1.default.trim);
};
exports.inString = inString;
const inNumber = (txt) => {
    let result = [];
    txt.split(',').map(item => {
        try {
            result.push(parseInt(lodash_1.default.trim(item)));
        }
        catch (_a) {
        }
    });
    return result;
};
exports.inNumber = inNumber;
const BOOLEAN_TEXT_IGNORE = new Set(['', 'null']);
const BOOLEAN_TEXT_FALSE = new Set(['0', 'false', '-1']);
const boolValue = (value) => {
    let bool = true;
    if (lodash_1.default.isString(value)) {
        let v = lodash_1.default.toLower(lodash_1.default.trim(value));
        if (BOOLEAN_TEXT_IGNORE.has(v))
            return;
        if (BOOLEAN_TEXT_FALSE.has(v))
            bool = false;
    }
    else {
        bool = !!value;
    }
    return bool;
};
exports.boolValue = boolValue;
