import type { TObject, Static } from '@sinclair/typebox';
import type { QuerySchema, WhereParam, WhereDefine, USchema } from '../../base/types';
import type { TableOptions } from '../../base/BaseView';
import type { SearchRequest, SearchResponse, QueryDslQueryContainer, Sort } from '@elastic/elasticsearch/lib/api/types';
import { BaseQuery } from './BaseQuery';
import type { ESQuery } from './define';
export declare abstract class BaseView<T extends TObject, ROW> extends BaseQuery {
    protected _index: string;
    protected abstract _EXECUTOR: ESQuery<T, ROW>;
    private _F2C;
    private _C2F;
    protected _CONFIG: {
        timeField: string;
        sort: Sort;
        mark: any;
        pageSize: number;
        FIELD_MAP: Map<string, USchema>;
        fields_exclude: any[];
        globalFilter: QueryDslQueryContainer;
    };
    protected _QUERY_CACHE: Map<string, WhereDefine>;
    /**
     * @param tableName Data table name, "${schemaName}.${tableName}"
     *  "${schemaName}." can be ignore with the default search_path.
     * @param schema The Object Schema, oor will not validate the value
     * @param options (Table/View) Options
     *
    */
    constructor(indexName: string, schema: T, options?: TableOptions);
    /**
     * @returns return Type dependson Constructor and Entity :
     *    Use View / Table ： Return Array< SearchHit<Entity> >
     *    Use FlatView ： Return Array< Entity >
    */
    all(): Promise<ROW[]>;
    private _query;
    /**
     * @see WhereCondition
     *    Use a WhereCondition Query Data
     * @returns return Type dependson Constructor and Entity :
     *    Use View / Table ： Return Array< SearchHit<Entity> >
     *    Use FlatView ： Return Array< Entity >
    */
    queryByCondition(condition?: WhereParam, query?: QuerySchema): Promise<ROW[]>;
    /**
     * @see QuerySchema
     *      Use a QuerySchema Query Data
     * @returns return Type dependson Constructor and Entity :
     *    Use View / Table ： Return Array< SearchHit<Entity> >
     *    Use FlatView ： Return Array< Entity >
    */
    query(query?: QuerySchema): Promise<ROW[]>;
    /**
     * Exec A Senctence
    */
    sql(request?: SearchRequest): Promise<SearchResponse<Static<T>>>;
    /**
     * @see QuerySchema
     *    Use a QuerySchema Query Data With Page
     *    this will return a object with {total:number,list:any[]}
     * @returns return Type dependson Constructor and Entity :
     *    property : list
     *    Use View / Table ： Return Array < SearchHit <Entity> >
     *    Use FlatView ： Return Array < Entity >
    */
    queryPager(query?: QuerySchema): Promise<{
        total: number;
        list: ROW[];
    }>;
    protected getField(key: string): string;
    protected getColumn(key: string): string;
    protected byField(field: string, value?: string | number | boolean): QueryDslQueryContainer;
    /**
     * Get A record form Table / View By Primary key.
     * This method will return All column. Even if the IGNORE column.
    */
    getById(id: string): Promise<Static<T>>;
    /**
     * Get A record form Table / View By Specify Field = value.
     * This method will return All column. Even if the IGNORE column.
     * Note : If result has multi records , return the first row
     *        Want return all records?  use `queryByField`
    */
    getByField(field: string, value: string | number): Promise<ROW>;
    /**
     * Get records form Table / View By Specify Property = value.
    */
    queryByField(field: string, value?: string | number | boolean): Promise<ROW[]>;
    private orderByLimit;
}
