import type { TObject, Static } from '@sinclair/typebox';
import type { QuerySchema, WhereParam } from '../../base/types';
import type { SearchHit } from '@elastic/elasticsearch/lib/api/types';
import { View } from './View';
export declare class Table<T extends TObject> extends View<T> {
    /**
     * Auto convert data
     * check row data while insert or update
     * */
    private checkEntity;
    /**
     * Delete documents by match {term:{filed,value}}
    */
    deleteByField(field: string, value: string | number | boolean): Promise<number>;
    /**
     * Delete documents by a QueryLike Schema
    */
    deleteByQuery(query: QuerySchema): Promise<number>;
    deleteByCondition(condition: WhereParam): Promise<number>;
    /**
     * Delete documents by DOCUMENT._id
     *  (Not id field in Source) : Source id can use deleteByField('id',xxx);
    */
    deleteById(id: string): Promise<number>;
    /**
     * Update a record,  by DOCUMENT._id
     * (Not id field in Source) : Source id can use updateByField(doc, 'id',xxx);
    */
    update(id: string, obj: Static<T>, useIndex?: boolean): Promise<number>;
    /**
     * Update documents by match {term:{filed,value}}
    */
    updateByField(obj: Static<T>, field: string, value: string | number | boolean): Promise<number>;
    updateByQuery(obj: Static<T>, query: QuerySchema): Promise<number>;
    updateByCondition(obj: Static<T>, condition?: WhereParam): Promise<number>;
    /**
     * Insert a record
    */
    add(object: Static<T>): Promise<SearchHit<Static<T>>>;
}
