"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.actions = exports.flatQuerys = exports.querys = void 0;
const lodash_1 = __importDefault(require("lodash"));
const Util_1 = require("../../base/Util");
const logSearch = (request) => {
    if (Util_1.ShowSql == null)
        return;
    let url = ['POST /', request.index, '/_search'];
    if (request._source_excludes && request._source_excludes.length) {
        url.push('?_source_excludes=');
        url.push(request._source_excludes.join(','));
    }
    (0, Util_1.ShowSql)(`${url.join('')} ${JSON.stringify(Object.assign(Object.assign({}, request), { _source_excludes: undefined, index: undefined }))}`);
};
const logExecuer = (method, url, param) => {
    if (Util_1.ShowSql == null)
        return;
    return (0, Util_1.ShowSql)(`${method} ${url} ${param != null ? JSON.stringify(param) : ''}`);
};
exports.querys = {
    query: (client, request) => __awaiter(void 0, void 0, void 0, function* () {
        logSearch(request);
        let result = yield client.search(request);
        return result.hits.hits;
    }),
    queryPager: (client, request) => __awaiter(void 0, void 0, void 0, function* () {
        logSearch(request);
        let total = 0;
        let result = yield client.search(request);
        if (lodash_1.default.isNumber(result.hits.total)) {
            total = result.hits.total;
        }
        else {
            total = result.hits.total.value;
        }
        return { total, list: result.hits.hits };
    }),
    getById: (client, index, id) => __awaiter(void 0, void 0, void 0, function* () {
        logExecuer('GET', '/' + index + '/_doc/' + id);
        try {
            return yield client.getSource({ id, index });
        }
        catch (_a) {
            return null;
        }
    }),
    getOne: (client, request) => __awaiter(void 0, void 0, void 0, function* () {
        let param = Object.assign(Object.assign({}, request), { from: 0, size: 1 });
        logSearch(param);
        const result = yield client.search(request);
        if (result.hits.hits.length) {
            return result.hits.hits[0];
        }
        return null;
    }),
};
exports.flatQuerys = Object.assign(Object.assign({}, exports.querys), { query: (client, request) => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield exports.querys.query(client, request);
        return result.map(item => item._source);
    }), queryPager: (client, request) => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield exports.querys.queryPager(client, request);
        return { total: result.total, list: result.list.map(item => item._source) };
    }) });
exports.actions = {
    add: (client, index, doc) => __awaiter(void 0, void 0, void 0, function* () {
        logExecuer('POST', '/' + index + '/_doc', doc);
        const result = yield client.index({ op_type: 'create', index, document: doc });
        if (result.result == 'created') {
            return {
                _id: result._id,
                _index: result._index,
                _version: result._version,
                _source: doc
            };
        }
        throw new Error();
    }),
    updateById: (client, index, id, entity, useIndex = false) => __awaiter(void 0, void 0, void 0, function* () {
        if (useIndex) {
            let indexRequest = { index, id, op_type: 'index', document: entity };
            logExecuer('PUT', '/' + index + '/_doc/' + id + '?optype=index', entity);
            const result = yield client.index(indexRequest);
            return result.result == 'updated' ? 1 : 0;
        }
        let updateRequest = { index, id, doc: entity };
        logExecuer('POST', '/' + index + '/_update/' + id, { doc: entity });
        const result = yield client.update(updateRequest);
        return result.result == 'updated' ? 1 : 0;
    }),
    updateByQuery: (client, index, param, script) => __awaiter(void 0, void 0, void 0, function* () {
        logExecuer('POST', '/' + index + '/_update_by_query', { script, query: param });
        const result = yield client.updateByQuery({ index, script, query: param });
        return result.updated;
    }),
    deleteById: (client, index, id) => __awaiter(void 0, void 0, void 0, function* () {
        logExecuer('DELETE', '/' + index + '/_doc/' + id);
        const result = yield client.delete({ id, index });
        return result.result == 'deleted' ? 1 : 0;
    }),
    deleteByQuery: (client, index, query) => __awaiter(void 0, void 0, void 0, function* () {
        logExecuer('DELETE', '/' + index + '/_delete_by_query', { query });
        const result = yield client.deleteByQuery({ index, query, });
        return result.deleted;
    }),
};
