"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.byField = exports.limit = exports.orderBy = exports.del = exports.update = exports.insert = exports.count = exports.select = void 0;
const lodash_1 = __importDefault(require("lodash"));
const Util_1 = require("../../base/Util");
const select = (table, fields = '*') => `SELECT ${fields || '*'} FROM \`${table}\` `;
exports.select = select;
const count = (table) => `SELECT count(0) AS total FROM \`${table}\``;
exports.count = count;
const insert = (table, obj) => {
    const fields = lodash_1.default.keys(obj);
    if (fields.length == 0) {
        throw new Error();
    }
    let query = [];
    let idx = [];
    let param = [];
    fields.map((field, i) => {
        let val = obj[field];
        if (val === null) {
            return;
        }
        query.push(field);
        idx.push("?");
        param.push(val);
    });
    return [`INSERT INTO \`${table}\` (${query.join(',')}) VALUES (${idx.join(',')}) ;`, param];
};
exports.insert = insert;
const update = (table, obj, key = 'id') => {
    const fields = lodash_1.default.keys(obj);
    if (fields.length == 0)
        throw new Error();
    let query = [];
    let param = [];
    let diff = 1;
    fields.map((field, i) => {
        // Not Allow Update Primary Key
        if (field == key) {
            diff = 0;
            return;
        }
        let val = obj[field];
        query.push(`${field} = ?`);
        param.push(val);
    });
    return [`UPDATE \`${table}\` SET ${query.join(',')}`, param];
};
exports.update = update;
const del = (table) => `DELETE FROM \`${table}\` `;
exports.del = del;
const BY_SET = new Set(['asc', 'desc']);
const orderBy = (ftc, ctf, query, default_sort) => {
    let by = 'desc';
    if (query) {
        if (query.order_) {
            if (query.by_ && BY_SET.has(query.by_))
                by = query.by_;
            if (ctf.has(query.order_)) {
                return `ORDER BY ${query.order_} ${by}`;
            }
            else if (ftc.has(query.order_)) {
                return `ORDER BY ${ftc.get(query.order_)} ${by}`;
            }
            return '';
        }
    }
    if (default_sort) {
        return `ORDER BY ${default_sort.order} ${default_sort.by}`;
    }
    return '';
};
exports.orderBy = orderBy;
const limit = (query, pageSize = Util_1.PAGE_SIZE) => {
    let start = lodash_1.default.has(query, 'start_') ? query.start_ : 0;
    let count = lodash_1.default.has(query, 'count_') ? query.count_ : pageSize;
    return `LIMIT ${count} OFFSET ${start}`;
};
exports.limit = limit;
const byField = (field, id) => [` ${field} = ? `, [id]];
exports.byField = byField;
