"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setup = exports.Table = exports.View = exports.UType = void 0;
const lodash_1 = __importDefault(require("lodash"));
const promise_1 = require("mysql2/promise");
const Util_1 = require("../base/Util");
const QueryBuilder_1 = require("../base/QueryBuilder");
const BaseView_1 = require("../base/BaseView");
const BaseTable_1 = require("../base/BaseTable");
const builder_1 = require("./basic/builder");
const where_1 = require("./basic/where");
const executor_1 = require("./basic/executor");
// Export Some useful global apis/types.
var Util_2 = require("../base/Util");
Object.defineProperty(exports, "UType", { enumerable: true, get: function () { return Util_2.UType; } });
const MY = { select: builder_1.select, count: builder_1.count, insert: builder_1.insert, delete: builder_1.del, update: builder_1.update, where: where_1.where, orderBy: builder_1.orderBy, limit: builder_1.limit, byField: builder_1.byField, };
const SelectField = (field, schema) => {
    const type = (0, QueryBuilder_1.getFieldType)(schema);
    if (type == 'boolean') {
        if (schema.column)
            return `IF(\`${schema.column}\`= 1, TRUE, FALSE) AS \`${field}\``;
        return `IF(\`${field}\`= 1, TRUE, FALSE) AS \`${field}\``;
    }
    if (schema.column)
        return `\`${schema.column}\` AS \`${field}\``;
    return '`' + field + '`';
};
class View extends BaseView_1.BaseView {
    constructor() {
        super(...arguments);
        this._DB_TYPE = 'mysql';
        this._BUILDER = MY;
        this._EXECUTOR = executor_1.executor;
    }
    init(schema, options) {
        let fields_query = [];
        let fields_get = [];
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            let select = SelectField(field, properties);
            fields_get.push(select);
            if (properties.ignore === true)
                return;
            fields_query.push(select);
        });
        this._CONFIG.fields_query = fields_query.join(',');
        this._CONFIG.fields_get = fields_get.join(',');
        const WHERE = [];
        if (this._CONFIG.mark) {
            let column = lodash_1.default.keys(this._CONFIG.mark)[0];
            WHERE.push({ field: column, value: this._CONFIG.mark[column], condition: '!=' });
        }
        if (options && options.globalCondition && options.globalCondition.length) {
            options.globalCondition.map(item => {
                let schema = this._CONFIG.FIELD_MAP.get(this._C2F.get(item.column));
                if (schema) {
                    WHERE.push(Object.assign(Object.assign({}, item), { type: (0, QueryBuilder_1.getFieldType)(schema) }));
                }
                else {
                    console.error(item);
                }
            });
        }
        this._CONFIG.WHERE_FIX = (0, where_1.fixWhere)(this._CONFIG.FIELD_MAP, WHERE);
    }
    /**
     * same arguments as mysql.query()
     * */
    exec(...args) {
        return this.getClient().query.call(this.getClient(), ...args);
    }
}
exports.View = View;
class Table extends BaseTable_1.BaseTable {
    constructor() {
        super(...arguments);
        this._DB_TYPE = 'mysql';
        this._BUILDER = MY;
        this._EXECUTOR = executor_1.executor;
    }
    init(schema, options) {
        let fields_query = [];
        let fields_get = [];
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            let select = SelectField(field, properties);
            fields_get.push(select);
            if (properties.ignore === true)
                return;
            fields_query.push(select);
        });
        this._CONFIG.fields_query = fields_query.join(',');
        this._CONFIG.fields_get = fields_get.join(',');
        const WHERE = [];
        if (this._CONFIG.mark) {
            let column = lodash_1.default.keys(this._CONFIG.mark)[0];
            WHERE.push({ field: column, value: this._CONFIG.mark[column], condition: '!=' });
        }
        if (options && options.globalCondition && options.globalCondition.length) {
            options.globalCondition.map(item => {
                let schema = this._CONFIG.FIELD_MAP.get(this._C2F.get(item.column));
                if (schema) {
                    WHERE.push(Object.assign(Object.assign({}, item), { type: (0, QueryBuilder_1.getFieldType)(schema) }));
                }
                else {
                    console.error(item);
                }
            });
        }
        this._CONFIG.WHERE_FIX = (0, where_1.fixWhere)(this._CONFIG.FIELD_MAP, WHERE);
    }
    add(object) {
        const _super = Object.create(null, {
            add: { get: () => super.add }
        });
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield _super.add.call(this, object);
            return yield this.getById(result['id']);
        });
    }
    /**
     * same arguments as mysql.query()
     * */
    exec(...args) {
        return this.getClient().query.call(this.getClient(), ...args);
    }
}
exports.Table = Table;
const setup = (settings, cb) => {
    let pool;
    if (lodash_1.default.isFunction(settings.provider)) {
        pool = settings.provider();
    }
    else {
        pool = (0, promise_1.createPool)(settings.provider);
    }
    (0, Util_1.setup)(Object.assign(Object.assign({}, settings), { provider: ['mysql', () => pool] }));
    return pool;
};
exports.setup = setup;
