"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setup = exports.Table = exports.View = exports.UType = void 0;
const lodash_1 = __importDefault(require("lodash"));
const pg_1 = require("pg");
const Util_1 = require("../base/Util");
const QueryBuilder_1 = require("../base/QueryBuilder");
const BaseView_1 = require("../base/BaseView");
const BaseTable_1 = require("../base/BaseTable");
const builder_1 = require("./basic/builder");
const where_1 = require("./basic/where");
const executor_1 = require("./basic/executor");
// Export Some useful global apis/types.
var Util_2 = require("../base/Util");
Object.defineProperty(exports, "UType", { enumerable: true, get: function () { return Util_2.UType; } });
const PG = { select: builder_1.select, count: builder_1.count, insert: builder_1.insert, delete: builder_1.del, update: builder_1.update, where: where_1.where, orderBy: builder_1.orderBy, limit: builder_1.limit, byField: builder_1.byField, };
class View extends BaseView_1.BaseView {
    constructor() {
        super(...arguments);
        this._DB_TYPE = 'pg';
        this._BUILDER = PG;
        this._EXECUTOR = executor_1.executor;
    }
    /**
     * 1. INIT  _CONFIG.fields_query  AND _CONFIG.fields_get
     * 5. Fix Where Condition (NEED BUILDER?)
    */
    init(schema, options) {
        let fields_query = [];
        let fields_get = [];
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            if (properties.column) {
                fields_get.push(`"${properties.column}" AS "${field}"`);
                if (properties.ignore === true) {
                    return;
                }
                fields_query.push(`"${properties.column}" AS "${field}"`);
            }
            else {
                fields_get.push('"' + field + '"');
                if (properties.ignore === true) {
                    return;
                }
                fields_query.push('"' + field + '"');
            }
        });
        this._CONFIG.fields_query = fields_query.join(',');
        this._CONFIG.fields_get = fields_get.join(',');
        const WHERE = [];
        if (this._CONFIG.mark) {
            let column = lodash_1.default.keys(this._CONFIG.mark)[0];
            WHERE.push({ field: column, value: this._CONFIG.mark[column], condition: '!=' });
        }
        if (options && options.globalCondition && options.globalCondition.length) {
            options.globalCondition.map(item => {
                let schema = this._CONFIG.FIELD_MAP.get(this._C2F.get(item.column));
                if (schema) {
                    WHERE.push(Object.assign(Object.assign({}, item), { type: (0, QueryBuilder_1.getFieldType)(schema) }));
                }
                else {
                    console.error(item);
                }
            });
        }
        this._CONFIG.WHERE_FIX = (0, where_1.fixWhere)(this._CONFIG.FIELD_MAP, WHERE);
    }
    /**
     * same arguments as pg.query()
     * */
    exec(...args) {
        return this.getClient().query.call(this.getClient(), ...args);
    }
}
exports.View = View;
class Table extends BaseTable_1.BaseTable {
    constructor() {
        super(...arguments);
        this._DB_TYPE = 'pg';
        this._BUILDER = PG;
        this._EXECUTOR = executor_1.executor;
    }
    init(schema, options) {
        let fields_query = [];
        let fields_get = [];
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            if (properties.column) {
                fields_get.push(`"${properties.column}" AS "${field}"`);
                if (properties.ignore === true) {
                    return;
                }
                fields_query.push(`"${properties.column}" AS "${field}"`);
            }
            else {
                fields_get.push('"' + field + '"');
                if (properties.ignore === true) {
                    return;
                }
                fields_query.push('"' + field + '"');
            }
        });
        this._CONFIG.fields_query = fields_query.join(',');
        this._CONFIG.fields_get = fields_get.join(',');
        const WHERE = [];
        if (this._CONFIG.mark) {
            let column = lodash_1.default.keys(this._CONFIG.mark)[0];
            WHERE.push({ field: column, value: this._CONFIG.mark[column], condition: '!=' });
        }
        if (options && options.globalCondition && options.globalCondition.length) {
            options.globalCondition.map(item => {
                let schema = this._CONFIG.FIELD_MAP.get(this._C2F.get(item.column));
                if (schema) {
                    WHERE.push(Object.assign(Object.assign({}, item), { type: (0, QueryBuilder_1.getFieldType)(schema) }));
                }
                else {
                    console.error(item);
                }
            });
        }
        this._CONFIG.WHERE_FIX = (0, where_1.fixWhere)(this._CONFIG.FIELD_MAP, WHERE);
    }
    /**
     * same arguments as pg.query()
     * */
    exec(...args) {
        return this.getClient().query.call(this.getClient(), ...args);
    }
}
exports.Table = Table;
const setup = (settings, cb) => {
    let pool;
    if (lodash_1.default.isFunction(settings.provider)) {
        pool = settings.provider();
    }
    else {
        pool = new pg_1.Pool(settings.provider);
        pool.connect(cb);
    }
    (0, Util_1.setup)(Object.assign(Object.assign({}, settings), { provider: ['pg', () => pool] }));
    return pool;
};
exports.setup = setup;
